@@ -9,9 +9,10 @@ use strict;
 use warnings;
 use base qw( IO::Async::Handle );
 
-our $VERSION = '0.50';
+our $VERSION = '0.52';
 
 use IO::Async::Handle;
+use IO::Async::OS;
 
 use Errno qw( EAGAIN EWOULDBLOCK );
 
@@ -304,7 +305,7 @@ is returned by the C<getaddrinfo> named resolver.
 
 Shortcut for passing a single address to listen on; it may be passed directly
 with this key, instead of in another array of its own. This should be in a
-format recognised by L<IO::Async::Loop>'s C<extract_addrinfo> method. See also
+format recognised by L<IO::Async::OS>'s C<extract_addrinfo> method. See also
 the C<EXAMPLES> section.
 
 =back
@@ -450,11 +451,11 @@ sub listen
       my ( $listenerr, $binderr, $sockopterr, $socketerr );
 
       foreach my $addr ( @$addrlist ) {
-         my ( $family, $socktype, $proto, $address ) = $loop->extract_addrinfo( $addr );
+         my ( $family, $socktype, $proto, $address ) = IO::Async::OS->extract_addrinfo( $addr );
 
          my $sock;
 
-         unless( $sock = $loop->socket( $family, $socktype, $proto ) ) {
+         unless( $sock = IO::Async::OS->socket( $family, $socktype, $proto ) ) {
             $socketerr = $!;
             $on_fail->( socket => $family, $socktype, $proto, $! ) if $on_fail;
             next;
@@ -587,7 +588,7 @@ sockets.
 =head2 Passing Plain Socket Addresses
 
 The C<addr> or C<addrs> parameters should contain a definition of a plain
-socket address in a form that the L<IO::Async::Loop> C<extract_addrinfo>
+socket address in a form that the L<IO::Async::OS> C<extract_addrinfo>
 method can use.
 
 This example shows how to use the C<Socket> functions to construct one for
