@@ -8,7 +8,7 @@ package IO::Async::Loop::Poll;
 use strict;
 use warnings;
 
-our $VERSION = '0.50';
+our $VERSION = '0.52';
 use constant API_VERSION => '0.49';
 
 use base qw( IO::Async::Loop );
@@ -17,7 +17,7 @@ use Carp;
 
 use IO::Poll qw( POLLIN POLLOUT POLLHUP POLLERR );
 
-use POSIX qw( EINTR );
+use POSIX qw( EINTR S_ISREG );
 
 # IO::Poll version 0.05 contain a bug whereby the ->remove method doesn't
 # properly clean up all the references to the handles. If the version we're
@@ -30,6 +30,10 @@ use constant _CAN_ON_HANGUP =>
    ( $^O eq "linux" ) ||
    ( $^O eq "freebsd" and do { no warnings 'numeric'; (POSIX::uname)[2] >= 8.0 } );
 
+# poll() on most platforms claims that ISREG files are always read- and
+# write-ready, but not on MSWin32. We need to fake this
+use constant FAKE_ISREG_READY => IO::Async::OS->HAVE_FAKE_ISREG_READY;
+
 =head1 NAME
 
 C<IO::Async::Loop::Poll> - use C<IO::Async> with C<poll(2)>
@@ -140,6 +144,9 @@ sub post_poll
       my $watch = $iowatches->{$fd} or next;
 
       my $events = $poll->events( $watch->[0] );
+      if( FAKE_ISREG_READY and $self->{fake_isreg}{$fd} ) {
+         $events |= $self->{fake_isreg}{$fd} & ( POLLIN|POLLOUT );
+      }
 
       # We have to test separately because kernel doesn't report POLLIN when
       # a pipe gets closed.
@@ -180,6 +187,8 @@ sub loop_once
 
    $self->_adjust_timeout( \$timeout );
 
+   $timeout = 0 if FAKE_ISREG_READY and keys %{ $self->{fake_isreg} };
+
    # Round up to nearest millisecond
    if( $timeout ) {
       my $mils = $timeout * 1000;
@@ -239,6 +248,10 @@ sub watch_io
    $params{on_write_ready} and $mask |= POLLOUT;
    $params{on_hangup}      and $mask |= POLLHUP;
 
+   if( FAKE_ISREG_READY and S_ISREG(stat $handle) ) {
+      $self->{fake_isreg}{$handle->fileno} = [ $handle, $mask ];
+   }
+
    $poll->mask( $handle, $mask ) if $mask != $curmask;
 }
 
@@ -261,6 +274,15 @@ sub unwatch_io
    $params{on_write_ready} and $mask &= ~POLLOUT;
    $params{on_hangup}      and $mask &= ~POLLHUP;
 
+   if( FAKE_ISREG_READY and S_ISREG(stat $handle) ) {
+      if( $mask ) {
+         $self->{fake_isreg}{$handle->fileno} = [ $handle, $mask ];
+      }
+      else {
+         delete $self->{fake_isreg}{$handle->fileno};
+      }
+   }
+
    $poll->mask( $handle, $mask ) if $mask != $curmask;
 }
 
