@@ -19,13 +19,15 @@ use Test::Refcount;
 
 use IO::Async::Test qw();
 
+use IO::Async::OS;
+
 use IO::File;
 use Fcntl qw( SEEK_SET );
 use POSIX qw( SIGTERM WIFEXITED WEXITSTATUS WIFSIGNALED WTERMSIG );
 use Socket qw( sockaddr_family AF_UNIX );
 use Time::HiRes qw( time );
 
-our $VERSION = '0.50';
+our $VERSION = '0.52';
 
 # Abstract Units of Time
 use constant AUT => $ENV{TEST_QUICK_TIMERS} ? 0.1 : 1;
@@ -155,11 +157,11 @@ Tests the Loop's ability to watch filehandles for IO readiness
 
 =cut
 
-use constant count_tests_io => 16;
+use constant count_tests_io => 17;
 sub run_tests_io
 {
    {
-      my ( $S1, $S2 ) = $loop->socketpair or die "Cannot create socket pair - $!";
+      my ( $S1, $S2 ) = IO::Async::OS->socketpair or die "Cannot create socket pair - $!";
       $_->blocking( 0 ) for $S1, $S2;
 
       my $readready  = 0;
@@ -189,6 +191,25 @@ sub run_tests_io
       # Ready $S1 to clear the data
       $S1->getline; # ignore return
 
+      $loop->unwatch_io(
+         handle => $S1,
+         on_read_ready => 1,
+      );
+
+      $loop->watch_io(
+         handle => $S1,
+         on_read_ready => sub { $readready = 1 },
+      );
+
+      $readready = 0;
+      $S2->syswrite( "more data\n" );
+
+      $loop->loop_once( 0.1 );
+
+      is( $readready, 1, '$readready after ->unwatch_io/->watch_io' );
+
+      $S1->getline; # ignore return
+
       $loop->watch_io(
          handle => $S1,
          on_write_ready => sub { $writeready = 1 },
@@ -225,7 +246,7 @@ sub run_tests_io
 
    # HUP of pipe - can be different to sockets on some architectures
    {
-      my ( $Prd, $Pwr ) = $loop->pipepair or die "Cannot pipepair - $!";
+      my ( $Prd, $Pwr ) = IO::Async::OS->pipepair or die "Cannot pipepair - $!";
       $_->blocking( 0 ) for $Prd, $Pwr;
 
       my $readready = 0;
@@ -255,7 +276,7 @@ sub run_tests_io
       $loop->_CAN_ON_HANGUP or skip "Loop cannot watch_io for on_hangup", 2;
 
       SKIP: {
-         my ( $S1, $S2 ) = $loop->socketpair or die "Cannot socketpair - $!";
+         my ( $S1, $S2 ) = IO::Async::OS->socketpair or die "Cannot socketpair - $!";
          $_->blocking( 0 ) for $S1, $S2;
 
          sockaddr_family( $S1->sockname ) == AF_UNIX or skip "Cannot reliably detect hangup condition on non AF_UNIX sockets", 1;
@@ -273,7 +294,7 @@ sub run_tests_io
          is( $hangup, 1, '$hangup after socket close' );
       }
 
-      my ( $Prd, $Pwr ) = $loop->pipepair or die "Cannot pipepair - $!";
+      my ( $Prd, $Pwr ) = IO::Async::OS->pipepair or die "Cannot pipepair - $!";
       $_->blocking( 0 ) for $Prd, $Pwr;
 
       my $hangup = 0;
@@ -291,7 +312,7 @@ sub run_tests_io
 
    # Check that combined read/write handlers can cancel each other
    {
-      my ( $S1, $S2 ) = $loop->socketpair or die "Cannot socketpair - $!";
+      my ( $S1, $S2 ) = IO::Async::OS->socketpair or die "Cannot socketpair - $!";
       $_->blocking( 0 ) for $S1, $S2;
 
       my $callcount = 0;
@@ -317,7 +338,7 @@ sub run_tests_io
    # Check that error conditions that aren't true read/write-ability are still
    # invoked
    {
-      my ( $S1, $S2 ) = $loop->socketpair( 'inet', 'dgram' ) or die "Cannot create AF_INET/SOCK_DGRAM connected pair - $!";
+      my ( $S1, $S2 ) = IO::Async::OS->socketpair( 'inet', 'dgram' ) or die "Cannot create AF_INET/SOCK_DGRAM connected pair - $!";
       $_->blocking( 0 ) for $S1, $S2;
       $S2->close;
 
@@ -379,6 +400,8 @@ sub run_tests_timer
    my $done = 0;
    # New watch/unwatch API
 
+   cmp_ok( abs( $loop->time - time ), "<", 0.1, '$loop->time gives the current time' );
+
    $loop->watch_time( after => 2 * AUT, code => sub { $done = 1; } );
 
    is_oneref( $loop, '$loop has refcount 1 after watch_time' );
@@ -426,20 +449,22 @@ sub run_tests_timer
       $loop->loop_once while !$done;
    } 0, 0.1, 'loop_once while waiting for negative interval timer';
 
-   my $doneA;
-   my $doneB;
+   {
+      my $done;
 
-   $id = $loop->watch_time( after => 1 * AUT, code => sub {
-      $loop->unwatch_time( $id ); undef $id;
-      $doneA++;
-   });
+      my $id;
+      $id = $loop->watch_time( after => 1 * AUT, code => sub {
+         $loop->unwatch_time( $id ); undef $id;
+      });
 
-   $loop->watch_time( after => 1.1 * AUT, code => sub { $doneB++ } );
+      $loop->watch_time( after => 1.1 * AUT, code => sub {
+         $done++;
+      });
 
-   $loop->loop_once( 1 * AUT ) for 1 .. 3;
+      wait_for { $done };
 
-   is( $doneA, 1, 'Self-cancelling timer still fires' );
-   is( $doneB, 1, 'Other timers still fire after self-cancelling one' );
+      is( $done, 1, 'Other timers still fire after self-cancelling one' );
+   }
 
    # Legacy enqueue/requeue/cancel API
    $done = 0;
@@ -460,21 +485,25 @@ sub run_tests_timer
       }
    } 1.5, 2.5, 'loop_once(5) while waiting for timer';
 
-   # Check that short delays are achievable in one ->loop_once call
-   foreach my $delay ( 0.001, 0.01, 0.1 ) {
-      my $done;
-      my $count = 0;
-      my $start = time;
+   SKIP: {
+      skip "Unable to handle sub-second timers accurately", 3 unless $loop->_CAN_SUBSECOND_ACCURATELY;
 
-      $loop->enqueue_timer( delay => $delay, code => sub { $done++ } );
+      # Check that short delays are achievable in one ->loop_once call
+      foreach my $delay ( 0.001, 0.01, 0.1 ) {
+         my $done;
+         my $count = 0;
+         my $start = time;
 
-      while( !$done ) {
-         $loop->loop_once( 1 );
-         $count++;
-         last if time - $start > 5; # bailout
+         $loop->enqueue_timer( delay => $delay, code => sub { $done++ } );
+
+         while( !$done ) {
+            $loop->loop_once( 1 );
+            $count++;
+            last if time - $start > 5; # bailout
+         }
+
+         is( $count, 1, "One ->loop_once(1) sufficient for a single $delay second timer" );
       }
-
-      is( $count, 1, "One ->loop_once(1) sufficient for a single $delay second timer" );
    }
 
    $cancelled_fired = 0;
