@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2012-2013 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2012 -- leonerd@leonerd.org.uk
 
 package IO::Async::OS;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.60_004';
+our $VERSION = '0.52';
 
 our @ISA = qw( IO::Async::OS::_Base );
 
@@ -40,8 +40,6 @@ use constant HAVE_FAKE_ISREG_READY => 0;
 
 # Do we have to select() for for evec to get connect() failures
 use constant HAVE_SELECT_CONNECT_EVEC => 0;
-# Ditto; do we have to poll() for POLLPRI to get connect() failures
-use constant HAVE_POLL_CONNECT_POLLPRI => 0;
 
 # Does connect() yield EWOULDBLOCK for nonblocking in progress?
 use constant HAVE_CONNECT_EWOULDBLOCK => 0;
@@ -49,22 +47,6 @@ use constant HAVE_CONNECT_EWOULDBLOCK => 0;
 # Can we rename() files that are open?
 use constant HAVE_RENAME_OPEN_FILES => 1;
 
-# Do we have IO::Socket::IP available?
-use constant HAVE_IO_SOCKET_IP => defined eval { require IO::Socket::IP };
-
-# Can we reliably watch for POSIX signals, including SIGCHLD to reliably
-# inform us that a fork()ed child has exit()ed?
-use constant HAVE_SIGNALS => 1;
-
-# Do we support POSIX-style true fork()ed processes at all?
-use constant HAVE_POSIX_FORK => !$ENV{IO_ASYNC_NO_FORK};
-# Can we potentially support threads? (would still need to 'require threads')
-use constant HAVE_THREADS => !$ENV{IO_ASYNC_NO_THREADS} &&
-   eval { require Config && $Config::Config{useithreads} };
-
-# Preferred trial order for built-in Loop classes
-use constant LOOP_BUILTIN_CLASSES => qw( Poll Select );
-
 =head1 NAME
 
 C<IO::Async::OS> - operating system abstractions for C<IO::Async>
@@ -75,33 +57,6 @@ This module acts as a class to provide a number of utility methods whose exact
 behaviour may depend on the type of OS it is running on. It is provided as a
 class so that specific kinds of operating system can override methods in it.
 
-As well as these support functions it also provides a number of constants, all
-with names beginning C<HAVE_> which describe various features that may or may
-not be available on the OS or perl build. Most of these are either hard-coded
-per OS, or detected at runtime.
-
-The following constants may be overridden by environment variables.
-
-=over 4
-
-=item * HAVE_POSIX_FORK
-
-True if the C<fork()> call has full POSIX semantics (full process separation).
-This is true on most OSes but false on MSWin32.
-
-This may be overridden to be false by setting the environment variable
-C<IO_ASYNC_NO_FORK>.
-
-=item * HAVE_THREADS
-
-True if C<ithreads> are available, meaning that the C<threads> module can be
-used. This depends on whether perl was built with threading support.
-
-This may be overridable to be false by setting the environment variable
-C<IO_ASYNC_NO_THREADS>.
-
-=back
-
 =cut
 
 =head2 $family = IO::Async::OS->getfamilybyname( $name )
@@ -160,21 +115,15 @@ sub socket
    my ( $family, $socktype, $proto ) = @_;
 
    croak "Cannot create a new socket without a family" unless $family;
-   # PF_UNSPEC and undef are both false
-   $family = $self->getfamilybyname( $family ) || AF_UNIX;
 
    # SOCK_STREAM is the most likely
-   $socktype = $self->getsocktypebyname( $socktype ) || SOCK_STREAM;
+   defined $socktype or $socktype = SOCK_STREAM;
 
    defined $proto or $proto = 0;
 
-   if( HAVE_IO_SOCKET_IP and ( $family == AF_INET || $family == AF_INET6() ) ) {
-      return IO::Socket::IP->new->socket( $family, $socktype, $proto );
-   }
-
    my $sock = eval {
       IO::Socket->new(
-         Domain => $family,
+         Domain => $family, 
          Type   => $socktype,
          Proto  => $proto,
       );
@@ -430,8 +379,6 @@ sub extract_addrinfo
 =item family => 'inet'
 
 Will pack an IP address and port number from keys called C<ip> and C<port>.
-If C<ip> is missing it will be set to "0.0.0.0". If C<port> is missing it will
-be set to 0.
 
 =cut
 
@@ -440,8 +387,8 @@ sub _extract_addrinfo_inet
    my $self = shift;
    my ( $ai ) = @_;
 
-   my $port = $ai->{port} || 0;
-   my $ip   = $ai->{ip}   || "0.0.0.0";
+   defined( my $port = $ai->{port} ) or croak "Expected 'port' for extract_addrinfo on family='inet'";
+   defined( my $ip   = $ai->{ip}   ) or croak "Expected 'ip' for extract_addrinfo on family='inet'";
 
    return pack_sockaddr_in( $port, inet_aton( $ip ) );
 }
@@ -449,9 +396,8 @@ sub _extract_addrinfo_inet
 =item family => 'inet6'
 
 Will pack an IP address and port number from keys called C<ip> and C<port>.
-If C<ip> is missing it will be set to "::". If C<port> is missing it will be
-set to 0. Optionally will also include values from C<scopeid> and C<flowinfo>
-keys if provided.
+Optionally will also include values from C<scopeid> and C<flowinfo> keys if
+provided.
 
 This will only work if a C<pack_sockaddr_in6> function can be found in
 C<Socket>
@@ -463,8 +409,9 @@ sub _extract_addrinfo_inet6
    my $self = shift;
    my ( $ai ) = @_;
 
-   my $port     = $ai->{port}     || 0;
-   my $ip       = $ai->{ip}       || "::";
+   defined( my $port = $ai->{port} ) or croak "Expected 'port' for extract_addrinfo on family='inet6'";
+   defined( my $ip   = $ai->{ip}   ) or croak "Expected 'ip' for extract_addrinfo on family='inet6'";
+
    my $scopeid  = $ai->{scopeid}  || 0;
    my $flowinfo = $ai->{flowinfo} || 0;
 
